﻿using isosoft.root;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace iDigitalRoman
{
	public class iRomanDictionarySequence : XObjectNotifier
	{
		int				_decimalSequence	= -1;	/// 1 = 1-9, 10 = 10-90, 100 = 100-900, 1000 = 1000-9000...
		int				_romanGroupOrder	= 1;
		List<string>	_romanSymbols		= new List<string>(10);

		public iRomanDictionarySequence()
		{

		}

		internal iRomanDictionarySequence(int seq, int romanGroupOrder)
		{
			_romanGroupOrder	= romanGroupOrder;
			DecimalSequence		= seq;
		}

		public string this[int index]
		{
			get { return _romanSymbols[index]; }
		}

		public string this[string romanNum]
		{
			get { return _romanSymbols.FirstOrDefault(i => i != null && i.Equals(romanNum, StringComparison.InvariantCultureIgnoreCase)); }
		}

		public int RomanGroup
		{
			get {  return _romanGroupOrder; }
			internal set
			{
				_romanGroupOrder = value;
				NotifyPropertyChanged(() => RomanGroup);
			}
		}

		public int RomanIndex(string romanNum)
		{
			return _romanSymbols.IndexOf(romanNum.ToUpper());
		}

		public int DecimalFactor
		{
			get
			{
				if(_decimalSequence < 0)
					return 0;

				return (int) Math.Round(Math.Log10(_decimalSequence));
			}
		}


		public int DecimalSequence
		{
			get {  return _decimalSequence; }
			internal set
			{
				//if(value == _decimalSequence)
				//	return;

				if(! ApplyDecimalSequence(value))
					return;

				_decimalSequence	= value;
				NotifyPropertyChanged(() => DecimalSequence);
				NotifyPropertyChanged(() => DecimalFactor);
			}
		}


		public List<string> BuildingBlockElements
		{
			get {  return _romanSymbols; }
		}

		private bool ApplyDecimalSequence(int value)
		{
			switch(value)
			{
				case 1:
					_romanSymbols	= new List<string>()
					{
						"I",			// 1
						"II",			// 2
						"III",			// 3
						"IV",			// 4
						"V",			// 5
						"VI",			// 6
						"VII",			// 7
						"VIII",			// 8
						"IX",			// 9
					};
					return true;

				case 10:
					_romanSymbols = new List<string>()
					{
						"X",			// 10
						"XX",			// 20
						"XXX",			// 30
						"XL",			// 40
						"L",			// 50
						"LX",			// 60
						"LXX",			// 70
						"LXXX",			// 80
						"XC",			// 90
					};
					return true;

				case 100:
					_romanSymbols = new List<string>()
					{
						"C",			// 100
						"CC",			// 200
						"CCC",			// 300
						"CD",			// 400
						"D",			// 500
						"DC",			// 600
						"DCC",			// 700
						"DCCC",			// 800
						"CM",			// 900
					};
					return true;

				case 1000:
					_romanSymbols = new List<string>()
					{
						"M",			// 1000
						"MM",			// 2000
						"MMM",			// 3000
						"MVM",			// 4000
						"VM",			// 5000
						"VMM",			// 6000
						"VMMM",			// 7000
						"VMMMM",		// 8000
						"MXM",			// 9000
					};
					return true;

				case 10000:
					_romanSymbols = new List<string>()
					{
						"XM",			// 10000
						"XMXM",			// 20000
						"XMXMXM",		// 30000
						"XMVXM",		// 40000
						"VXM",			// 50000
						"VXMXM",		// 60000
						"VXMXMXM",		// 70000
						"VXMXMXMXM",	// 80000
						"CMXM",			// 90000
					};
					return true;

				case 100000:
					_romanSymbols = new List<string>()
					{
						"CM",			// 100000
						"CMCM",			// 200000
						"CMCMCM",		// 300000
						"CMVCM",		// 400000
						"VCM",			// 500000
						"VCMCM",		// 600000
						"VCMCMCM",		// 700000
						"VCMCMCMCM",	// 800000
						"MMCM",			// 900000
					};
					return true;


				case 1000000:
					_romanSymbols = new List<string>()
					{
						"U",			// 1000000
						"UU",			// 2000000
						"UUU",			// 3000000
						"UW",			// 4000000
						"W",			// 5000000
						"WU",			// 6000000
						"WUU",			// 7000000
						"WUUU",			// 8000000
						"UY",			// 9000000
					};
					return true;
				default:
					return false;
			}
		}
	}
}
