﻿using isosoft.root;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace iDigitalRoman
{

	public class iRomanNumberDictionary : XObjectNotifier
	{
		static iRomanNumberDictionary		_instance			= null;
		public const int					MaxRomanGroup		= 7;

		List<iRomanDictionarySequence>		_buildingBlocks		= new List<iRomanDictionarySequence>();
		bool								_conversionInError	= false;
		int?								_errorRecurseLevel	= 0,
											_errorGroup			= 0;
		string								_errorRomanNumber;

		/// pedagogic history
		List<iRoman2DecimalHistoryItem>		_roman2DecimalHitory	= new List<iRoman2DecimalHistoryItem>();

		public static iRomanNumberDictionary Instance
		{
			get
			{
				if(_instance == null)
					_instance = new iRomanNumberDictionary();

				return _instance;
			}
		}

		public static iRomanNumberDictionary GetInstance()
		{
			return Instance;
		}



		iRomanNumberDictionary()
		{
			init_instance();
		}

		void init_instance()
		{
			int[]		sequences	= { 1, 10, 100, 1000, 10000, 100000, 1000000, };
			int			group		= 1;

			foreach(int seq in sequences)
			{
				_buildingBlocks.Add(new iRomanDictionarySequence(seq, group++));
			}
		}

		public int? ErrorRecurseLevel
		{
			get {  return _conversionInError ? _errorRecurseLevel : null; }
		}

		public int? ErrorGroup
		{
			get {  return _conversionInError ? _errorGroup : null; }
		}


		public bool ConversionInError
		{
			get { return _conversionInError; }
			protected set
			{
				if(value ==  _conversionInError)
					return;

				_conversionInError = value;
				NotifyPropertyChanged(() => ConversionInError);
				NotifyPropertyChanged(() => ErrorRecurseLevel);
				NotifyPropertyChanged(() => ErrorGroup);
			}
		}

		public List<iRoman2DecimalHistoryItem> Roman2DecHistory
		{
			get {  return _roman2DecimalHitory; }
		}

		public string Number2Roman(int number)
		{
			string		strnum,
						romanString,
						nextRomanString;
			int			numberIndex,
						sequenceIndex,
						len,
						recurseNumber;

			strnum		= number.ToString();
			romanString	= strnum;

			if(number > 9999999)
			{
				SetError(true, 0, 0, null);
				return "??";
			}

			_conversionInError	= false;

			/// get left most digit value
			if(!int.TryParse(strnum.Substring( 0, 1), out numberIndex))
				return "";

			len					= strnum.Length;
			sequenceIndex		= len - 1;
			
			if(numberIndex == 0)
				romanString	= "";
			else
				romanString = _buildingBlocks[sequenceIndex ][numberIndex - 1]?? "";

			if(sequenceIndex < len)
			{
				/// find next roman sequence
				nextRomanString	= strnum.Substring(len - sequenceIndex);

				if(int.TryParse(nextRomanString, out recurseNumber))
				{
					nextRomanString	= Number2Roman(recurseNumber);
					romanString		+= nextRomanString;
				}
			}

			return romanString;
		}


		public static bool IsValidRomanNumber(string romanString)
		{
			if(string.IsNullOrWhiteSpace(romanString))
				return false;

			List<char>	validChars	= new List<char>() { 'i', 'v', 'x', 'l', 'c', 'd', 'm', 'u', 'w', 'y', };

			string		lower	= romanString.Trim().ToLower();

			foreach(char c in lower)
			{
				if(validChars.IndexOf(c) < 0)
					return false;
			}

			return true;
		}

		public string RomanNumberNotFound
		{
			get {  return _conversionInError ? _errorRomanNumber : null; }
		}

		void SetError(bool isInError, int recurseLevel, int group, string numberNotFound)
		{
			ConversionInError	= isInError;
			_errorRecurseLevel	= recurseLevel;
			_errorGroup			= group;
			_errorRomanNumber	= numberNotFound;

			_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("error", numberNotFound, 0, group));
			NotifyPropertyChanged(() => Roman2DecHistory);
		}

		public int Roman2Number(string romanstring, int startAtGroup, int recurseLevel = 0)
		{
			/// if this a recurse call and the group is least = error
			if(recurseLevel > 0 && startAtGroup < 1)
			{
				SetError(true, recurseLevel, startAtGroup, romanstring);
				return 0;
			}

			/// do we have correct arguments
			if(startAtGroup < 1 || startAtGroup > MaxRomanGroup ||  ! IsValidRomanNumber(romanstring))
			{
				SetError(true, recurseLevel, startAtGroup, romanstring);
				return 0;
			}

			if(recurseLevel <= 0)
				_roman2DecimalHitory.Clear();

			ConversionInError	= false;
			romanstring			= romanstring.Trim();

			_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("●start●", romanstring, 0, startAtGroup));

			int							startGroup		= startAtGroup;
			string						recurseString	= romanstring;
			int							len				= romanstring.Length;
			iRomanDictionarySequence	seq				= FindSequence4String(romanstring, startGroup);
			int							finalValue		= 0,
										num;

			/// already found a sequence for the entire string?
			if(seq != null)
			{
				/// sequence group should be less or equal than start
				/// i.e. we first find the highest decimal value
				if(seq.RomanGroup <= startAtGroup)
				{
					finalValue	= seq.DecimalSequence * (seq.RomanIndex(romanstring) + 1);

					_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("●found●", romanstring, finalValue, seq.RomanGroup));
					return finalValue;
				}

				SetError(true, recurseLevel, startAtGroup, romanstring);
				return 0;
			}

			List<int>			values			= new List<int>();
			
			/// try to find the highest value starting at this group
			while(seq == null && len > 1)
			{
				recurseString	= recurseString.Substring(0, len - 1);
				seq				= FindSequence4String(recurseString, startGroup);

				len				= recurseString.Length;
				_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("searching", recurseString, 0, startGroup));
			}

			if(seq == null || startGroup < 1)
			{
				SetError(true, recurseLevel, startAtGroup, romanstring);
				return 0;
			}

			/// add the value found in the current group
			num		= seq.DecimalSequence * (seq.RomanIndex(recurseString) + 1);
			values.Add(num);
			_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("found intermediate", recurseString, num, seq.RomanGroup));

			/// search in the next lower group
			startGroup		= seq.RomanGroup;

			/// take the next remainder string
			recurseString	= romanstring.Substring(recurseString.Length);

			/// find the remainder string in the lower groups (recurse)
			if(! _conversionInError)
			{
				_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("recurse lower group", recurseString, 0, startGroup - 1));
				num		= Roman2Number(recurseString, startGroup - 1, recurseLevel + 1);
				values.Add(num);
			}

			/// sum the found values
			foreach(int v in values)
				finalValue	+= v;

			_roman2DecimalHitory.Add(new iRoman2DecimalHistoryItem("●final value●", romanstring, finalValue, startAtGroup));

			NotifyPropertyChanged(() => Roman2DecHistory);

			return finalValue;
		}


		iRomanDictionarySequence FindSequence4String(string romanString, int groupOrder)
		{
			if(string.IsNullOrEmpty(romanString))
				return null;

			return _buildingBlocks.FirstOrDefault(i => i[romanString] != null && i.RomanGroup <= groupOrder);
		}
	}

	public class iRoman2DecimalHistoryItem
	{
		public string		Message			{  get; set; }
		public string		RomanString		{  get; set; }
		public int			CurrentGroup	{  get; set; }
		public int			FoundDecimal	{  get; set; }

		public iRoman2DecimalHistoryItem(string message, string roman, int number, int group)
		{
			Message			= message;
			RomanString		= roman;
			FoundDecimal	= number;
			CurrentGroup	= group;
		}
	}
}
