﻿using isosoft.root;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace iDigitalRoman.view_models
{
	public class iRomanViewModel : XObjectNotifier
	{
		static iRomanViewModel		_instance;

		public static iRomanViewModel Instance
		{
			get
			{
				if(_instance == null)
					_instance = new iRomanViewModel();

				return _instance;
			}
		}

		public static iRomanViewModel GetInstance()
		{
			return Instance;
		}


		protected int		_decimalNumber;
		protected string	_romanNumber;
		protected bool		_updatingDecimal		= false,
							_updatingRoman			= false;

		public iRomanViewModel() : base()
		{

		}

		public bool ConversionInError
		{
			get {  return iRomanNumberDictionary.Instance.ConversionInError; }
		}

		public int? ErrorRecurseLevel
		{
			get {  return iRomanNumberDictionary.Instance.ErrorRecurseLevel; }
		}

		public int? ErrorGroup
		{
			get {  return iRomanNumberDictionary.Instance.ErrorGroup; }
		}

		public string RomanNumberNotFound
		{
			get {  return iRomanNumberDictionary.Instance.RomanNumberNotFound; }
		}

		public List<iRoman2DecimalHistoryItem> Roman2DecHistory
		{
			get {  return iRomanNumberDictionary.Instance.Roman2DecHistory; }
		}

		void NotifyConversionChanged()
		{
			NotifyPropertyChanged(() => DecimalNumber);
			NotifyPropertyChanged(() => RomanNumber);
			NotifyPropertyChanged(() => ConversionInError);
			NotifyPropertyChanged(() => ErrorRecurseLevel);
			NotifyPropertyChanged(() => ErrorGroup);
			NotifyPropertyChanged(() => RomanNumberNotFound);
			NotifyPropertyChanged(() => Roman2DecHistory);
		}

		public int DecimalNumber
		{
			get {  return _decimalNumber; }
			set
			{
				//if(value == _decimalNumber)
				//	return;

				_updatingDecimal	= true;
				_decimalNumber		= value;
				ConvertDecimal(value);
				_updatingDecimal	= false;

				NotifyConversionChanged();
			}
		}

		private void ConvertDecimal(int value)
		{
			if(_updatingDecimal)
			{
				iRomanNumberDictionary.Instance.Roman2DecHistory.Clear();
				NotifyPropertyChanged(() => Roman2DecHistory);

				_romanNumber	= iRomanNumberDictionary.Instance.Number2Roman(value);
			}
		}


		public string RomanNumber
		{
			get {  return _romanNumber; }
			set
			{
				//if(value == _romanNumber)
				//	return;

				_updatingRoman	= true;
				_romanNumber	= value;
				ConvertRoman(value);
				_updatingRoman	= false;

				NotifyConversionChanged();
			}
		}

		private void ConvertRoman(string value)
		{
			if(_updatingRoman)
			{
				iRomanNumberDictionary.Instance.Roman2DecHistory.Clear();
				NotifyPropertyChanged(() => Roman2DecHistory);
				_decimalNumber = iRomanNumberDictionary.Instance.Roman2Number(value, iRomanNumberDictionary.MaxRomanGroup);
			}
		}
	}

	public class iRomanViewModelDesignTime : iRomanViewModel
	{
		public iRomanViewModelDesignTime() : base()
		{
			init_instance();
		}

		private void init_instance()
		{
			this.DecimalNumber	= 45;
		}
	}

	public class iDecimalRomanList: XObjectListNotifier<iDecimalRoman>
	{
		static iDecimalRomanList		_instance;

		string							_romanFilter;

		public string RomanFilter
		{
			get {  return _romanFilter; }
			set
			{
				if(value == _romanFilter)
					return;

				_romanFilter	= value;
				NotifyPropertyChanged(() => RomanFilter);
				NotifyPropertyChanged(() => FilteredList);
			}
		}


		public static iDecimalRomanList Instance
		{
			get
			{
				if(_instance == null)
					_instance	= new iDecimalRomanList();

				return _instance;
			}
		}

		public static iDecimalRomanList GetInstance()
		{
			return Instance;
		}


		protected iDecimalRomanList()
		{
			init_instance();
		}

		private void init_instance()
		{
			this.Clear();

			for(int num = 1; num <= 2050; num++)
			{
				this.Add(new view_models.iDecimalRoman(num, iRomanNumberDictionary.Instance.Number2Roman(num)));
			}
		}

		public List<iDecimalRoman> FilteredList
		{
			get
			{
				if(string.IsNullOrEmpty(_romanFilter) || ! iRomanNumberDictionary.IsValidRomanNumber(_romanFilter))
					return this;

				return this.Where(item => item.Roman.ToLower().Contains(_romanFilter.ToLower())).ToList();
			}
		}
	}


	public class iDecimalRoman : XObjectNotifier
	{
		int			_num;
		string		_roman;

		public iDecimalRoman(int num, string roman)
		{
			_num	= num;
			_roman	= roman;
		}

		public int Decimal
		{
			get {  return _num; }
		}

		public string Roman
		{
			get {  return _roman; }
		}
	}
}
